@@ -12,21 +12,33 @@ Spyder API Mixins.
 """
 
 # Standard library imports
+from collections import OrderedDict
 from typing import Any, Optional, Dict
 
 # Third party imports
-from qtpy.QtCore import Qt
-from qtpy.QtWidgets import QSizePolicy, QToolBar, QWidget, QToolButton
+from qtpy.QtCore import QPoint, Qt
+from qtpy.QtGui import QIcon, QImage, QPainter, QPixmap
+from qtpy.QtSvg import QSvgRenderer
+from qtpy.QtWidgets import (
+    QApplication, QMainWindow, QSizePolicy, QToolBar, QWidget, QToolButton
+)
 
 # Local imports
-from spyder.api.config.mixins import SpyderConfigurationObserver
+from spyder.api.config.mixins import (
+    SpyderConfigurationAccessor,
+    SpyderConfigurationObserver
+)
 from spyder.api.exceptions import SpyderAPIError
+from spyder.api.shortcuts import SpyderShortcutsMixin
 from spyder.api.widgets.menus import SpyderMenu
+from spyder.api.widgets.toolbars import SpyderToolbar
 from spyder.config.manager import CONF
 from spyder.utils.icon_manager import ima
+from spyder.utils.image_path_manager import get_image_path
 from spyder.utils.qthelpers import create_action, create_toolbutton
 from spyder.utils.registries import (
     ACTION_REGISTRY, MENU_REGISTRY, TOOLBAR_REGISTRY, TOOLBUTTON_REGISTRY)
+from spyder.utils.stylesheet import PANES_TOOLBAR_STYLESHEET
 
 
 class SpyderToolButtonMixin:
@@ -37,7 +49,7 @@ class SpyderToolButtonMixin:
     def create_toolbutton(self, name, text=None, icon=None,
                           tip=None, toggled=None, triggered=None,
                           autoraise=True, text_beside_icon=False,
-                          section=None, option=None):
+                          section=None, option=None, register=True):
         """
         Create a Spyder toolbutton.
         """
@@ -63,7 +75,7 @@ class SpyderToolButtonMixin:
             id_=name,
             plugin=self.PLUGIN_NAME,
             context_name=self.CONTEXT_NAME,
-            register_toolbutton=True
+            register_toolbutton=register
         )
         toolbutton.name = name
 
@@ -137,29 +149,47 @@ class SpyderToolbarMixin:
     """
 
     def add_item_to_toolbar(self, action_or_widget, toolbar, section=None,
-                            before=None):
+                            before=None, before_section=None):
         """
         If you provide a `before` action, the action will be placed before this
         one, so the section option will be ignored, since the action will now
         be placed in the same section as the `before` action.
         """
-        toolbar.add_item(action_or_widget, section=section, before=before)
+        toolbar.add_item(action_or_widget, section=section, before=before,
+                         before_section=before_section)
 
-    def create_stretcher(self):
+    def create_stretcher(self, id_=None):
         """
         Create a stretcher widget to be used in a Qt toolbar.
         """
-        stretcher = QWidget()
+        stretcher = QWidget(self)
         stretcher.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+
+        if id_ is not None:
+            stretcher.ID = id_
         return stretcher
 
-    def create_toolbar(self, name: str) -> QToolBar:
+    def create_toolbar(
+        self,
+        name: str,
+        register: bool = True
+    ) -> SpyderToolbar:
         """
         Create a Spyder toolbar.
+
+        Parameters
+        ----------
+        name: str
+            Name of the toolbar to create.
+        register: bool
+            Whether to register the toolbar in the global registry.
         """
-        toolbar = QToolBar(self)
-        TOOLBAR_REGISTRY.register_reference(
-            toolbar, name, self.PLUGIN_NAME, self.CONTEXT_NAME)
+        toolbar = SpyderToolbar(self, name)
+        toolbar.setStyleSheet(str(PANES_TOOLBAR_STYLESHEET))
+        if register:
+            TOOLBAR_REGISTRY.register_reference(
+                toolbar, name, self.PLUGIN_NAME, self.CONTEXT_NAME
+            )
         return toolbar
 
     def get_toolbar(self, name: str, context: Optional[str] = None,
@@ -236,35 +266,99 @@ class SpyderMenuMixin:
 
         menu.add_action(action_or_menu, section=section, before=before)
 
-    def create_menu(self, name, text=None, icon=None):
+    def _create_menu(
+        self,
+        menu_id: str,
+        parent: Optional[QWidget] = None,
+        title: Optional[str] = None,
+        icon: Optional[QIcon] = None,
+        reposition: Optional[bool] = True,
+        register: bool = True,
+        min_width: Optional[int] = None,
+        MenuClass=SpyderMenu
+    ) -> SpyderMenu:
         """
-        Create a menu.
+        Create a SpyderMenu or a subclass of it.
 
-        Parameters
-        ----------
-        name: str
-            Unique str identifier.
-        text: str or None
-            Localized text string.
-        icon: QIcon or None
-            Icon to use for the menu.
-
-        Return: QMenu
-            Return the created menu.
+        Notes
+        -----
+        * This method must only be used directly to generate a menu that is a
+          subclass of SpyderMenu.
+        * Refer to the documentation for `SpyderMenu` to learn about its args.
         """
-        from spyder.api.widgets.menus import SpyderMenu
+        if register:
+            menus = getattr(self, '_menus', None)
+            if menus is None:
+                self._menus = OrderedDict()
+
+            if menu_id in self._menus:
+                raise SpyderAPIError(
+                    'Menu name "{}" already in use!'.format(menu_id)
+                )
+
+        menu = MenuClass(
+            parent=self if parent is None else parent,
+            menu_id=menu_id,
+            title=title,
+            min_width=min_width,
+            reposition=reposition
+        )
 
-        menu = SpyderMenu(parent=self, title=text)
         if icon is not None:
             menu.menuAction().setIconVisibleInMenu(True)
             menu.setIcon(icon)
 
-        MENU_REGISTRY.register_reference(
-            menu, name, self.PLUGIN_NAME, self.CONTEXT_NAME)
+        if register:
+            MENU_REGISTRY.register_reference(
+                menu, menu_id, self.PLUGIN_NAME, self.CONTEXT_NAME
+            )
+            self._menus[menu_id] = menu
+
         return menu
 
-    def get_menu(self, name: str, context: Optional[str] = None,
-                 plugin: Optional[str] = None) -> SpyderMenu:
+    def create_menu(
+        self,
+        menu_id: str,
+        title: Optional[str] = None,
+        icon: Optional[QIcon] = None,
+        reposition: Optional[bool] = True,
+        register: bool = True
+    ) -> SpyderMenu:
+        """
+        Create a menu for Spyder.
+
+        Parameters
+        ----------
+        menu_id: str
+            Unique str identifier for the menu.
+        title: str or None
+            Localized text string for the menu.
+        icon: QIcon or None
+            Icon to use for the menu.
+        reposition: bool, optional (default True)
+            Whether to vertically reposition the menu due to its padding.
+        register: bool
+            Whether to register the menu in the global registry.
+
+        Returns
+        -------
+        SpyderMenu
+            The created menu.
+        """
+        return self._create_menu(
+            menu_id=menu_id,
+            title=title,
+            icon=icon,
+            reposition=reposition,
+            register=register
+        )
+
+    def get_menu(
+        self,
+        name: str,
+        context: Optional[str] = None,
+        plugin: Optional[str] = None
+    ) -> SpyderMenu:
         """
         Return a menu by name, plugin and context.
 
@@ -289,7 +383,7 @@ class SpyderMenuMixin:
         ------
         KeyError
             If either of `name`, `context` or `plugin` keys do not exist in the
-            toolbar registry.
+            menu registry.
         """
         plugin = self.PLUGIN_NAME if plugin is None else plugin
         context = self.CONTEXT_NAME if context is None else context
@@ -347,11 +441,12 @@ class SpyderActionMixin:
     # other it refers to a section of the configuration (or the widget
     # name where it is applied).
     def create_action(self, name, text, icon=None, icon_text='', tip=None,
-                      toggled=None, triggered=None, shortcut_context=None,
+                      toggled=None, triggered=None, data=None,
+                      shortcut_context=None,
                       context=Qt.WidgetWithChildrenShortcut, initial=None,
                       register_shortcut=False, section=None, option=None,
                       parent=None, register_action=True, overwrite=False,
-                      context_name=None):
+                      context_name=None, menurole=None):
         """
         name: str
             unique identifiable name for the action
@@ -371,6 +466,8 @@ class SpyderActionMixin:
             behave like a checkbox.
         triggered: callable
             The callable to use when triggering this action.
+        data: Any
+            Data to be set on the action.
         shortcut_context: str
             Set the `str` context of the shortcut.
         context: Qt.ShortcutContext
@@ -400,7 +497,9 @@ class SpyderActionMixin:
             Name of the context that holds the action in case of registration.
             The combination of `name` and `context_name` is unique so trying
             to register an action with the same `name` and `context_name` will
-            cause a warning unless `overwrite` is set to `True`
+            cause a warning unless `overwrite` is set to `True`.
+        menurole: QAction.MenuRole, optional
+            Menu role for the action (it only has effect on macOS).
 
         Notes
         -----
@@ -436,6 +535,7 @@ class SpyderActionMixin:
             tip=tip,
             toggled=toggled,
             triggered=triggered,
+            data=data,
             context=context,
             section=section,
             option=option,
@@ -444,7 +544,8 @@ class SpyderActionMixin:
             context_name=(
                 self.CONTEXT_NAME if context_name is None else context_name),
             register_action=register_action,
-            overwrite=overwrite
+            overwrite=overwrite,
+            menurole=menurole
         )
         action.name = name
         if icon_text:
@@ -495,7 +596,7 @@ class SpyderActionMixin:
         ------
         KeyError
             If either of `name`, `context` or `plugin` keys do not exist in the
-            toolbar registry.
+            action registry.
         """
         plugin = self.PLUGIN_NAME if plugin is None else plugin
         context = self.CONTEXT_NAME if context is None else context
@@ -550,15 +651,17 @@ class SpyderActionMixin:
         raise NotImplementedError('')
 
 
-class SpyderWidgetMixin(SpyderActionMixin, SpyderMenuMixin,
-                        SpyderConfigurationObserver, SpyderToolButtonMixin):
+class SpyderWidgetMixin(
+    SpyderActionMixin,
+    SpyderConfigurationObserver,
+    SpyderMenuMixin,
+    SpyderToolbarMixin,
+    SpyderToolButtonMixin,
+    SpyderShortcutsMixin,
+):
     """
     Basic functionality for all Spyder widgets and Qt items.
 
-    This mixin does not include toolbar handling as that is limited to the
-    application with the coreui plugin or the PluginMainWidget for dockable
-    plugins.
-
     This provides a simple management of widget options, as well as Qt helpers
     for defining the actions a widget provides.
     """
@@ -602,3 +705,107 @@ class SpyderWidgetMixin(SpyderActionMixin, SpyderMenuMixin,
         change.
         """
         pass
+
+
+class SpyderMainWindowMixin:
+    """
+    Mixin with additional functionality for the QMainWindow's used in Spyder.
+    """
+
+    def _is_on_visible_screen(self: QMainWindow):
+        """Detect if the window is placed on a visible screen."""
+        x, y = self.geometry().x(), self.geometry().y()
+        qapp = QApplication.instance()
+        current_screen = qapp.screenAt(QPoint(x, y))
+
+        if current_screen is None:
+            return False
+        else:
+            return True
+
+    def move_to_primary_screen(self: QMainWindow):
+        """Move the window to the primary screen if necessary."""
+        if self._is_on_visible_screen():
+            return
+
+        qapp = QApplication.instance()
+        primary_screen_geometry = qapp.primaryScreen().availableGeometry()
+        x, y = primary_screen_geometry.x(), primary_screen_geometry.y()
+
+        if self.isMaximized():
+            self.showNormal()
+
+        self.move(QPoint(x, y))
+
+        # With this we want to maximize only the Spyder main window and not the
+        # plugin ones, which usually are not maximized.
+        if not hasattr(self, 'is_window_widget'):
+            self.showMaximized()
+
+
+class SvgToScaledPixmap(SpyderConfigurationAccessor):
+    """
+    Mixin to transform an SVG to a QPixmap that is scaled according to the
+    factor set by users in Preferences.
+    """
+
+    def svg_to_scaled_pixmap(self, svg_file, rescale=None, in_package=True):
+        """
+        Transform svg to a QPixmap that is scaled according to the factor set
+        by users in Preferences.
+
+        Parameters
+        ----------
+        svg_file: str
+            Name of or path to the svg file.
+        rescale: float, optional
+            Rescale pixmap according to a factor between 0 and 1.
+        in_package: bool, optional
+            Get svg from the `images` folder in the Spyder package.
+        """
+        if in_package:
+            image_path = get_image_path(svg_file)
+
+        if self.get_conf('high_dpi_custom_scale_factor', section='main'):
+            scale_factors = self.get_conf(
+                'high_dpi_custom_scale_factors',
+                section='main'
+            )
+            scale_factor = float(scale_factors.split(":")[0])
+        else:
+            scale_factor = 1
+
+        # Get width and height
+        pm = QPixmap(image_path)
+        width = pm.width()
+        height = pm.height()
+
+        # Rescale but preserving aspect ratio
+        if rescale is not None:
+            aspect_ratio = width / height
+            width = int(width * rescale)
+            height = int(width / aspect_ratio)
+
+        # Paint image using svg renderer
+        image = QImage(
+            int(width * scale_factor), int(height * scale_factor),
+            QImage.Format_ARGB32_Premultiplied
+        )
+        image.fill(0)
+        painter = QPainter(image)
+        renderer = QSvgRenderer(image_path)
+        renderer.render(painter)
+        painter.end()
+
+        # This is also necessary to make the image look good for different
+        # scale factors
+        if scale_factor > 1.0:
+            image.setDevicePixelRatio(scale_factor)
+
+        # Create pixmap out of image
+        final_pm = QPixmap.fromImage(image)
+        final_pm = final_pm.copy(
+            0, 0, int(width * scale_factor), int(height * scale_factor)
+        )
+
+        return final_pm
