@@ -19,9 +19,12 @@ from spyder.config.base import running_in_ci
 from spyder.config.utils import is_anaconda
 from spyder.plugins.ipythonconsole.tests.conftest import get_conda_test_env
 from spyder.utils.conda import (
-    add_quotes, find_conda, get_conda_activation_script, get_conda_env_path,
-    get_conda_root_prefix, get_list_conda_envs, get_list_conda_envs_cache,
-    get_spyder_conda_channel)
+    find_conda,
+    get_conda_root_prefix,
+    get_list_conda_envs,
+    get_list_conda_envs_cache,
+    get_spyder_conda_channel,
+)
 
 if not is_anaconda():
     pytest.skip("Requires conda to be installed", allow_module_level=True)
@@ -32,27 +35,6 @@ else:
     TEST_PYEXEC = '/miniconda/envs/foobar/bin/python'
 
 
-def test_add_quotes():
-    output = add_quotes('/some path/with spaces')
-    assert output == '"/some path/with spaces"'
-
-    output = add_quotes('/some-path/with-no-spaces')
-    assert output == '/some-path/with-no-spaces'
-
-
-def test_get_conda_activation_script():
-    output = get_conda_activation_script()
-    assert os.path.exists(output)
-
-
-def test_get_conda_env_path():
-    output = get_conda_env_path(TEST_PYEXEC)
-    if os.name == 'nt':
-        assert output == 'c:/miniconda/envs/foobar'
-    else:
-        assert output == '/miniconda/envs/foobar'
-
-
 def test_get_conda_root_prefix():
     output = get_conda_root_prefix(TEST_PYEXEC)
     if os.name == 'nt':
@@ -89,7 +71,7 @@ def test_get_list_conda_envs():
     output = get_list_conda_envs()
 
     expected_envs = ['base', 'jedi-test-env', 'spytest-ž', 'test']
-    expected_envs = ['conda: ' + env for env in expected_envs]
+    expected_envs = ['Conda: ' + env for env in expected_envs]
 
     assert set(expected_envs) == set(output.keys())
 
